/*
 * This file is part of essential (http://essential.craftforge.net).
 *
 *     Essential is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU Lesser Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     Essential is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (c) 2011 Christian Bick.
 */


package net.craftforge.essential.context.servlet;

import net.craftforge.essential.controller.Request;
import net.craftforge.essential.controller.constants.HttpMethod;
import net.craftforge.essential.controller.utils.CaseInsensitiveMap;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * An servlet implementation of a controller request which normalizes the data of an http servlet.
 *
 * @author Christian Bick
 * @since 20.02.2011
 */
public class HttpServletRequestImpl implements Request {

    private HttpServletRequest request;

    /**
     * Creates a controller request from a HTTP servlet request.
     *
     * @param request The HTTP servlet request
     */
    public HttpServletRequestImpl(HttpServletRequest request) {
        this.request = request;    
    }

    /**
     * {@inheritDoc}
     */
    public String getHttpMethod() {
        return request.getMethod();
    }

    /**
     * {@inheritDoc}
     */
    public String getUrlInfoPart() {
        return request.getPathInfo();
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public Map<String, String> getHeaders() {
        Map<String, String> headers = new CaseInsensitiveMap<String>();
        for (String headerName : (List<String>)Collections.list(request.getHeaderNames())) {
            headers.put(headerName, request.getHeader(headerName));
        }
        return headers;
    }

    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
    public Map<String, String[]> getParameters() {
        return request.getParameterMap();
    }

    /**
     * {@inheritDoc}
     */
    public InputStream getBodyInputStream() {
        if (getHttpMethod().equals(HttpMethod.PUT)
                || getHttpMethod().equals(HttpMethod.POST)) {
            try {
                return request.getInputStream();
            } catch (IOException e) {
                throw new RuntimeException("Unable to resolve request body input stream", e);
            }
        }
        return null;
    }
}
